
static char rcsid[] =
	"$Id: xpvm.c,v 2.310 1994/07/18 18:18:21 kohl Exp $";

#define DO_GLOBALS

#include "xpvm.h"

Tk_IdleProc *recv_event_proc();

/* Command Routines */

int define_trace_mask_proc();
int trace_file_status_proc();
int reset_trace_file_proc();
int trace_overwrite_proc();
int get_hosts_menu_list();
int fix_help_line_cmd();
int load_bitmap_file();
int output_file_proc();
int reset_views_proc();
int strip_label_cmd();
int trace_file_proc();
int trace_mask_proc();
int title_info_proc();
int get_tasks_proc();
int reset_pvm_proc();
int playback_proc();
int define_arch();
int hosts_proc();
int tasks_proc();
int quit_proc();
int halt_proc();
int pad_cmd();

/* MAIN */

extern int main();

int *tclDummyMainPtr = (int *) main;

Tcl_AppInit( itp )
Tcl_Interp *itp;
{
	/* Save Interpreter */

	interp = itp;

	/* Get Main Window */

	fprintf( stderr, "Create main window\n" );
	Top = Tk_MainWindow( interp );
	fprintf( stderr, "Created main window\n" );

	if ( Top == NULL )
		return( TCL_ERROR );

	Disp = Tk_Display( Top );
	fprintf( stderr, "Display setup\n" );

	/* Initialize Program Constants */

	if ( read_args() == TCL_ERROR )
		return( TCL_ERROR );

	/* Initialize Program Constants & Structs */

	fprintf( stderr, "program_init\n" );
	if ( program_init() == TCL_ERROR )
		return( TCL_ERROR );

	/* Initialize Modules */

	fprintf( stderr, "program_init over\n" );
	if ( tcl_init() == TCL_ERROR )
		return( TCL_ERROR );

	fprintf( stderr, "tcl_init over\n" );
	if ( pvm_init() == TCL_ERROR )
		return( TCL_ERROR );

	fprintf( stderr, "pvm_init over\n" );
	if ( window_init() == TCL_ERROR )
		return( TCL_ERROR );

	fprintf( stderr, "window_init over\n" );
	/* Set Up User-Specific Startup File */

	tcl_RcFileName = "~/.xpvmrc";

	/* Return to tkMain to Start Tk_MainLoop() */

	return( TCL_OK );
}

read_args()
{
	char tmp[1024];

	char *argv_str;

	int i, j, k;
	int do_usage;
	int len;

	argv_str = Tcl_GetVar( interp, "argv", TCL_GLOBAL_ONLY );

	if ( Tcl_SplitList( interp, argv_str, &Argc, &Argv ) == TCL_ERROR )
		return( TCL_ERROR );

	HOST_NAME = (char *) NULL;

	HOSTFILE = (char *) NULL;

	Nflag = 0;
	Xflag = 0;

	/* Qflag = 0; */

	eflag = 0;
	tflag = 0;
	vflag = 0;

	do_usage = 0;

	for ( i=0 ; i < Argc ; i++ )
	{
		if ( Argv[i][0] == '-' )
		{
			k = i + 1;

			len = strlen( Argv[i] );

			for ( j=0 ; j < len ; j++ )
			{
				switch ( Argv[i][j] )
				{
					case 'H': usage(); break;

					case 'N':
					{
						if ( HOST_NAME != NULL )
							free( HOST_NAME );

						HOST_NAME = copy_str( Argv[k++] );

						Nflag++;

						break;
					}

					case 'T':
					{
						sprintf( tmp, "set trace_file \"%s\"",
							Argv[k++] );

						Tcl_Eval( interp, tmp );

						break;
					}

					case 'S': dump_sddf_headers(); break;

					case 'X': Xflag++; break;

					/* case 'Q': Qflag++; break; */

					case 'e': eflag++; break;

					case 't': tflag++; break;

					case 'v': vflag++; break;

					case '-': break;

					default:
					{
						printf( "Unknown Option -%c\n", Argv[i][j] );

						do_usage++;

						break;
					}
				}
			}

			i = k - 1;
		}

		/* Hostfile */
		else
		{
			if ( HOSTFILE != NULL )
				free( HOSTFILE );

			HOSTFILE = copy_str( Argv[i] );
		}
	}

	if ( do_usage )
		usage();

	return( TCL_OK );
}

usage()
{
	printf( "\nusage:  xpvm [ hostfile ] " );
	printf( "[ -T trace ] [ -N name ] [ -HSetv ]\n\n" );

	printf( "where:\n" );
	printf( "------\n" );

	printf( "hostfile   = Alternate XPVM Hostfile (~/.xpvm_hosts)\n" );

	printf( "-T trace   = Use \"trace\" as Default Trace File\n" );

	printf( "-N name\t   = Use \"name\" as Local Network Hostname\n" );

	printf( "-H\t   = Print This Help Information\n" );
	printf( "-S\t   = Dump SDDF Trace File Headers (and exit)\n" );

	/* printf( "-Q\t   = Use Cached Group Servers\n" ); */

	printf( "-e\t   = Dump Raw PVM Event Text\n" );
	printf( "-t\t   = Text Mode, No Animation in Views\n" );
	printf( "-v\t   = Verbose Operation\n" );

	printf( "\n" );

	exit( 0 );
}

program_init()
{
	struct passwd *pw;

	char hname[1024];
	char tmp[2048];

	char *getenv();

	char *xpvm_dir;
	char *home;

	int i;

	/*
	 * Signals Set in pvm_init() - after pvmd started 
	 */

	/* Get Hostname */

	if ( HOST_NAME == NULL )
	{
		if ( gethostname( hname, 1024 ) == -1 )
		{
			perror( "Getting Hostname" );

			HOST_NAME = (char *) NULL;
		}

		else
			HOST_NAME = copy_str( hname );
	}

	HOST_ALIAS = host_alias_str( HOST_NAME );

	/* Get Home Directory */

	home = getenv("HOME");

	if ( home == NULL )
	{
		if ( (pw = getpwuid( getuid() )) != NULL )
			home = pw->pw_dir;

		else
			home = "/";

		HOME_DIR = copy_str( home );
	}

	else
		HOME_DIR = home;

	/* Get XPVM Main Directory */

	xpvm_dir = getenv("XPVM_ROOT");

	if ( xpvm_dir == NULL )
	{
		sprintf( tmp, "%s", XPVM_DEFAULT_DIR );

		if ( tmp[0] != '/' )
			sprintf( tmp, "%s/%s", HOME_DIR, XPVM_DEFAULT_DIR );

		XPVM_DIR = copy_str( tmp );
	}

	else
		XPVM_DIR = xpvm_dir;

	/* Check Hostfile */

	if ( HOSTFILE == NULL )
	{
		sprintf( tmp, "%s", DEFAULT_HOSTFILE );

		if ( tmp[0] != '/' )
			sprintf( tmp, "%s/%s", HOME_DIR, DEFAULT_HOSTFILE );
		
		HOSTFILE = copy_str( tmp );
	}

	/* Messages List */

	MSG_LIST = (MSG) NULL;

	NMSGS = 0;

	/* Tasks List */

	TASK_LIST = (TASK) NULL;

	NTASKS = 0;

	/* Hosts List */

	HOST_LIST = (HOST) NULL;

	NHOSTS = 0;

	/* Architectures List */

	ARCH_LIST = (ARCH) NULL;

	NARCHS = 0;

	/* Trace Mask Index List */

	TM_INDEX_LIST = (TRACE_MASK_INDEX) NULL;
	TM_INDEX_LAST = (TRACE_MASK_INDEX) NULL;

	SET_TRACE_MASK_INDEX( "ADDHOSTS",	TEV_ADDHOSTS0 );
	SET_TRACE_MASK_INDEX( "DELHOSTS",	TEV_DELHOSTS0 );
	SET_TRACE_MASK_INDEX( "CONFIG",		TEV_CONFIG0 );
	SET_TRACE_MASK_INDEX( "MSTAT",		TEV_MSTAT0 );
	SET_TRACE_MASK_INDEX( "ARCHCODE",	TEV_ARCHCODE0 );
	SET_TRACE_MASK_INDEX( "HOSTSYNC",	TEV_HOSTSYNC0 );

	SET_TRACE_MASK_INDEX( "BARRIER",	TEV_BARRIER0 );
	SET_TRACE_MASK_INDEX( "BCAST",		TEV_BCAST0 );
	SET_TRACE_MASK_INDEX( "DELETE",		TEV_DELETE0 );
	SET_TRACE_MASK_INDEX( "GETINST",	TEV_GETINST0 );
	SET_TRACE_MASK_INDEX( "GETTID",		TEV_GETTID0 );
	SET_TRACE_MASK_INDEX( "GSIZE",		TEV_GSIZE0 );
	SET_TRACE_MASK_INDEX( "INSERT",		TEV_INSERT0 );
	SET_TRACE_MASK_INDEX( "JOINGROUP",	TEV_JOINGROUP0 );
	SET_TRACE_MASK_INDEX( "LOOKUP",		TEV_LOOKUP0 );
	SET_TRACE_MASK_INDEX( "LVGROUP",	TEV_LVGROUP0 );

	SET_TRACE_MASK_INDEX( "MYTID",		TEV_MYTID0 );
	SET_TRACE_MASK_INDEX( "PARENT",		TEV_PARENT0 );
	SET_TRACE_MASK_INDEX( "SPAWN",		TEV_SPAWN0 );
	SET_TRACE_MASK_INDEX( "SENDSIG",	TEV_SENDSIG0 );
	SET_TRACE_MASK_INDEX( "KILL",		TEV_KILL0 );
	SET_TRACE_MASK_INDEX( "EXIT",		TEV_EXIT0 );

	SET_TRACE_MASK_INDEX( "MCAST",	TEV_MCAST0 );
	SET_TRACE_MASK_INDEX( "SEND",	TEV_SEND0 );
	SET_TRACE_MASK_INDEX( "RECV",	TEV_RECV0 );
	SET_TRACE_MASK_INDEX( "NRECV",	TEV_NRECV0 );
	SET_TRACE_MASK_INDEX( "RECVF",	TEV_RECVF0 );
	SET_TRACE_MASK_INDEX( "PSEND",	TEV_PSEND0 );
	SET_TRACE_MASK_INDEX( "PRECV",	TEV_PRECV0 );
	SET_TRACE_MASK_INDEX( "TRECV",	TEV_TRECV0 );
	SET_TRACE_MASK_INDEX( "PROBE",	TEV_PROBE0 );

	SET_TRACE_MASK_INDEX( "GATHER",		TEV_GATHER0 );
	SET_TRACE_MASK_INDEX( "SCATTER",	TEV_SCATTER0 );
	SET_TRACE_MASK_INDEX( "REDUCE",		TEV_REDUCE0 );

	SET_TRACE_MASK_INDEX( "INITSEND",	TEV_INITSEND0 );
	SET_TRACE_MASK_INDEX( "MKBUF",		TEV_MKBUF0 );
	SET_TRACE_MASK_INDEX( "BUFINFO",	TEV_BUFINFO0 );
	SET_TRACE_MASK_INDEX( "FREEBUF",	TEV_FREEBUF0 );
	SET_TRACE_MASK_INDEX( "GETRBUF",	TEV_GETRBUF0 );
	SET_TRACE_MASK_INDEX( "SETRBUF",	TEV_SETRBUF0 );
	SET_TRACE_MASK_INDEX( "GETSBUF",	TEV_GETSBUF0 );
	SET_TRACE_MASK_INDEX( "SETSBUF",	TEV_SETSBUF0 );
	SET_TRACE_MASK_INDEX( "GETMWID",	TEV_GETMWID0 );
	SET_TRACE_MASK_INDEX( "SETMWID",	TEV_SETMWID0 );

	SET_TRACE_MASK_INDEX( "PKBYTE",		TEV_PKBYTE0 );
	SET_TRACE_MASK_INDEX( "PKCPLX",		TEV_PKCPLX0 );
	SET_TRACE_MASK_INDEX( "PKDCPLX",	TEV_PKDCPLX0 );
	SET_TRACE_MASK_INDEX( "PKDOUBLE",	TEV_PKDOUBLE0 );
	SET_TRACE_MASK_INDEX( "PKFLOAT",	TEV_PKFLOAT0 );
	SET_TRACE_MASK_INDEX( "PKINT",		TEV_PKINT0 );
	SET_TRACE_MASK_INDEX( "PKUINT",		TEV_PKUINT0 );
	SET_TRACE_MASK_INDEX( "PKLONG",		TEV_PKLONG0 );
	SET_TRACE_MASK_INDEX( "PKULONG",	TEV_PKULONG0 );
	SET_TRACE_MASK_INDEX( "PKSHORT",	TEV_PKSHORT0 );
	SET_TRACE_MASK_INDEX( "PKUSHORT",	TEV_PKUSHORT0 );
	SET_TRACE_MASK_INDEX( "PKSTR",		TEV_PKSTR0 );
	SET_TRACE_MASK_INDEX( "PACKF",		TEV_PACKF0 );

	SET_TRACE_MASK_INDEX( "UPKBYTE",	TEV_UPKBYTE0 );
	SET_TRACE_MASK_INDEX( "UPKCPLX",	TEV_UPKCPLX0 );
	SET_TRACE_MASK_INDEX( "UPKDCPLX",	TEV_UPKDCPLX0 );
	SET_TRACE_MASK_INDEX( "UPKDOUBLE",	TEV_UPKDOUBLE0 );
	SET_TRACE_MASK_INDEX( "UPKFLOAT",	TEV_UPKFLOAT0 );
	SET_TRACE_MASK_INDEX( "UPKINT",		TEV_UPKINT0 );
	SET_TRACE_MASK_INDEX( "UPKUINT",	TEV_UPKUINT0 );
	SET_TRACE_MASK_INDEX( "UPKLONG",	TEV_UPKLONG0 );
	SET_TRACE_MASK_INDEX( "UPKULONG",	TEV_UPKULONG0 );
	SET_TRACE_MASK_INDEX( "UPKSHORT",	TEV_UPKSHORT0 );
	SET_TRACE_MASK_INDEX( "UPKUSHORT",	TEV_UPKUSHORT0 );
	SET_TRACE_MASK_INDEX( "UPKSTR",		TEV_UPKSTR0 );
	SET_TRACE_MASK_INDEX( "UNPACKF",	TEV_UNPACKF0 );

	SET_TRACE_MASK_INDEX( "PSTAT",		TEV_PSTAT0 );
	SET_TRACE_MASK_INDEX( "TASKS",		TEV_TASKS0 );
	SET_TRACE_MASK_INDEX( "GETOPT",		TEV_GETOPT0 );
	SET_TRACE_MASK_INDEX( "SETOPT",		TEV_SETOPT0 );
	SET_TRACE_MASK_INDEX( "TIDTOHOST",	TEV_TIDTOHOST0 );
	SET_TRACE_MASK_INDEX( "GETFDS",		TEV_GETFDS0 );

	SET_TRACE_MASK_INDEX( "START_PVMD",	TEV_START_PVMD0 );
	SET_TRACE_MASK_INDEX( "REG_HOSTER",	TEV_REG_HOSTER0 );
	SET_TRACE_MASK_INDEX( "REG_RM",		TEV_REG_RM0 );
	SET_TRACE_MASK_INDEX( "REG_TASKER",	TEV_REG_TASKER0 );
	SET_TRACE_MASK_INDEX( "CATCHOUT",	TEV_CATCHOUT0 );
	SET_TRACE_MASK_INDEX( "NOTIFY",		TEV_NOTIFY0 );
	SET_TRACE_MASK_INDEX( "GETTMASK",	TEV_GETTMASK0 );
	SET_TRACE_MASK_INDEX( "SETTMASK",	TEV_SETTMASK0 );
	SET_TRACE_MASK_INDEX( "PERROR",		TEV_PERROR0 );
	SET_TRACE_MASK_INDEX( "TICKLE",		TEV_TICKLE0 );
	SET_TRACE_MASK_INDEX( "VERSION",	TEV_VERSION0 );
	SET_TRACE_MASK_INDEX( "HALT",		TEV_HALT0 );

	SET_TRACE_MASK_INDEX( "NEWTASK",	TEV_NEWTASK );
	SET_TRACE_MASK_INDEX( "ENDTASK",	TEV_ENDTASK );
	SET_TRACE_MASK_INDEX( "SPNTASK",	TEV_SPNTASK );

	/* PVM Trace Event Status */

	for ( i=0 ; i < TRACE_MAX ; i++ )
		TEV_TRACE[i] = 0;

	/* PVM Event Types */

	TEV_TYPE[ TEV_ADDHOSTS0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_BARRIER0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_BCAST0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_BUFINFO0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_CONFIG0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_DELETE0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_DELHOSTS0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_FREEBUF0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GETFDS0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_GETINST0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GETOPT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_GETRBUF0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GETSBUF0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GETTID0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_GSIZE0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_HALT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_INITSEND0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_INSERT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_JOINGROUP0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_KILL0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_LOOKUP0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_LVGROUP0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_MCAST0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_MKBUF0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_MSTAT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_MYTID0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_NOTIFY0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_NRECV0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PARENT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PERROR0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PKBYTE0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PKCPLX0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PKDCPLX0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PKDOUBLE0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PKFLOAT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PKINT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PKUINT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PKLONG0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PKULONG0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PKSHORT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PKUSHORT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PKSTR0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PROBE0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PSTAT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_RECVF0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_SEND0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_SENDSIG0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_SETOPT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_SETRBUF0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_SETSBUF0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_SPAWN0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_START_PVMD0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_TASKS0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_TICKLE0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_TIDTOHOST0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKBYTE0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKCPLX0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKDCPLX0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKDOUBLE0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKFLOAT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKINT0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKUINT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKLONG0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKULONG0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKSHORT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKUSHORT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UPKSTR0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_VERSION0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_REG_HOSTER0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_REG_RM0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_REG_TASKER0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_ARCHCODE0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_CATCHOUT0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GETMWID0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GETTMASK0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_HOSTSYNC0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_PACKF0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_PSEND0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_REDUCE0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_SETMWID0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_SETTMASK0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_UNPACKF0 ]		= ENTRY_TEV;
	TEV_TYPE[ TEV_GATHER0 ]			= ENTRY_TEV;
	TEV_TYPE[ TEV_SCATTER0 ]		= ENTRY_TEV;

	TEV_TYPE[ TEV_ADDHOSTS1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_BARRIER1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_BCAST1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_BUFINFO1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_CONFIG1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_DELETE1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_DELHOSTS1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_FREEBUF1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GETFDS1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_GETINST1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GETOPT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_GETRBUF1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GETSBUF1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GETTID1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_GSIZE1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_HALT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_INITSEND1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_INSERT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_JOINGROUP1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_KILL1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_LOOKUP1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_LVGROUP1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_MCAST1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_MKBUF1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_MSTAT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_MYTID1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_NOTIFY1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_NRECV1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PARENT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PERROR1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PKBYTE1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PKCPLX1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PKDCPLX1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PKDOUBLE1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PKFLOAT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PKINT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PKUINT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PKLONG1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PKULONG1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PKSHORT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PKUSHORT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PKSTR1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PRECV1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PROBE1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PSTAT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_RECV1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_RECVF1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_SEND1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_SENDSIG1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_SETOPT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_SETRBUF1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_SETSBUF1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_SPAWN1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_START_PVMD1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_TASKS1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_TICKLE1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_TIDTOHOST1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_TRECV1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_UPKBYTE1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKCPLX1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKDCPLX1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKDOUBLE1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKFLOAT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKINT1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_UPKUINT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKLONG1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKULONG1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKSHORT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKUSHORT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UPKSTR1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_VERSION1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_REG_HOSTER1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_REG_RM1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_REG_TASKER1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_ARCHCODE1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_CATCHOUT1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GETMWID1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GETTMASK1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_HOSTSYNC1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_PACKF1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_PSEND1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_REDUCE1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_SETMWID1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_SETTMASK1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_UNPACKF1 ]		= EXIT_TEV;
	TEV_TYPE[ TEV_GATHER1 ]			= EXIT_TEV;
	TEV_TYPE[ TEV_SCATTER1 ]		= EXIT_TEV;

	TEV_TYPE[ TEV_RECV0 ]			= BLOCK_TEV;
	TEV_TYPE[ TEV_PRECV0 ]			= BLOCK_TEV;
	TEV_TYPE[ TEV_TRECV0 ]			= BLOCK_TEV;

	TEV_TYPE[ TEV_EXIT0 ]			= IGNORE_TEV;
	TEV_TYPE[ TEV_EXIT1 ]			= IGNORE_TEV;

	TEV_TYPE[ TEV_NEWTASK ]			= IGNORE_TEV;
	TEV_TYPE[ TEV_SPNTASK ]			= IGNORE_TEV;
	TEV_TYPE[ TEV_ENDTASK ]			= IGNORE_TEV;

	TEV_TYPE[ TRACE_OUTPUT ]		= IGNORE_TEV;
	TEV_TYPE[ TRACE_HOST_ADD ]		= IGNORE_TEV;
	TEV_TYPE[ TRACE_HOST_DEL ]		= IGNORE_TEV;
	TEV_TYPE[ TRACE_HOST_SYNC ]		= IGNORE_TEV;

	/* Initialize Time Vars */

	PROCESS_COUNTER_OVERWRITE = 4;
	PROCESS_COUNTER_PLAYBACK = 6;

	RECV_TIMER_OVERWRITE = 40;
	RECV_TIMER_PLAYBACK = 100;

	CURRENT_TIME.tv_sec = -1;
	CURRENT_TIME.tv_usec = -1;

	BASE_TIME.tv_sec = -1;
	BASE_TIME.tv_usec = -1;

	TRACE_TIME.tv_sec = -1;
	TRACE_TIME.tv_usec = -1;

	/* Initialize Trace Globals */

	TRACE_FILE_STATUS = TRACE_FILE_OVERWRITE;

	PROCESS_COUNTER_INTERVAL = PROCESS_COUNTER_OVERWRITE;
	RECV_TIMER_INTERVAL = RECV_TIMER_OVERWRITE;

	TRACE_OVERWRITE_FLAG = FALSE;

	TRACE_STATUS = TRACE_FWD;

	TRACE_DO_SPAWN = FALSE;

	TRACE_PENDING = 0;

	TRACE_ACTIVE = FALSE;

	strcpy( TRACE_FILE, "" );

	TRACE_OUT = (FILE *) NULL;
	TRACE_IN = (FILE *) NULL;

	strcpy( OUTPUT_FILE, "" );

	OUTPUT_FP = (FILE *) NULL;

	TRACE_GROUP_TASKS = FALSE;

	/* Misc Globals */

	GROUPS_ALIVE = FALSE;

	return( TCL_OK );
}

tcl_init()
{
	/* Initialize TCL / TK */

	if ( Tcl_Init( interp ) == TCL_ERROR )
		return( TCL_ERROR );
	fprintf( stderr, "Tcl_Init over\n" );

	if ( Tk_Init( interp ) == TCL_ERROR )
		return( TCL_ERROR );

	fprintf( stderr, "Tk_Init over\n" );
	/* Create TCL Commands for Action Routines */

	/* Main Console Commands */

	Tcl_CreateCommand( interp, "reset_trace_file",
		reset_trace_file_proc, (ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 1 over\n" );
	Tcl_CreateCommand( interp, "reset_views", reset_views_proc,
		(ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 2 over\n" );
	Tcl_CreateCommand( interp, "reset_pvm", reset_pvm_proc,
		(ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 3 over\n" );
	Tcl_CreateCommand( interp, "pvm_hosts", hosts_proc,
		(ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 4 over\n" );
	Tcl_CreateCommand( interp, "pvm_tasks", tasks_proc,
		(ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 5 over\n" );
	Tcl_CreateCommand( interp, "pvm_quit", quit_proc,
		(ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 6 over\n" );
	Tcl_CreateCommand( interp, "pvm_halt", halt_proc,
		(ClientData) NULL, (vfp) NULL );

	/* Trace Playback Commands */

	fprintf( stderr, "tcl_init create command 7 over\n" );
	Tcl_CreateCommand( interp, "trace_file_status_handle",
		trace_file_status_proc, (ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 8 over\n" );
	Tcl_CreateCommand( interp, "trace_overwrite_result",
		trace_overwrite_proc, (ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "trace_file_handle", trace_file_proc,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "pvm_trace_mask", trace_mask_proc,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "trace_rewind", playback_proc,
		(ClientData) TRACE_REWIND, (vfp) NULL );

	Tcl_CreateCommand( interp, "trace_stop", playback_proc,
		(ClientData) TRACE_STOP, (vfp) NULL );

	Tcl_CreateCommand( interp, "trace_fwd", playback_proc,
		(ClientData) TRACE_FWD, (vfp) NULL );

	Tcl_CreateCommand( interp, "trace_fwdstep", playback_proc,
		(ClientData) TRACE_FWDSTEP, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 9 over\n" );
	/* Menu & Miscellaneous Commands */

	Tcl_CreateCommand( interp, "define_trace_mask",
		define_trace_mask_proc, (ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "output_file_handle", output_file_proc,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "load_bitmap_file", load_bitmap_file,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "pvm_get_tasks", get_tasks_proc,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "fix_help_line", fix_help_line_cmd,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "strip_label", strip_label_cmd,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "title_info", title_info_proc,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "get_hosts_menu_list",
		get_hosts_menu_list, (ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "define_arch", define_arch,
		(ClientData) NULL, (vfp) NULL );

	Tcl_CreateCommand( interp, "pad", pad_cmd,
		(ClientData) NULL, (vfp) NULL );

	fprintf( stderr, "tcl_init create command 10 over\n" );
	return( TCL_OK );
}

pvm_init()
{
	char tmp[1024];

	char *av[4];
	int ac;

	int inum;
	int se;
	int cc;
	int i;

	/* Start PVM (if necessary) */

	av[0] = (char *) NULL;

	ac = 0;

	if ( HOSTFILE != NULL )
		av[ac++] = copy_str( HOSTFILE );

	if ( Nflag )
	{
		sprintf( tmp, "-n%s", HOST_NAME );

		av[ac++] = copy_str( tmp );
	}

	se = pvm_setopt( PvmAutoErr, 0 );

	cc = pvm_start_pvmd( ac, av, FALSE );

	if ( cc < 0 )
	{
		if ( cc == PvmDupHost )
		{
			printf( "Connecting to pvmd already running... " );
			fflush( stdout );

			NEW_PVM = FALSE;
		}

		else
		{
			pvm_perror( "Can't Start PVM" );

			exit( 1 );
		}
	}

	else
	{
		printf( "New pvmd started... " );
		fflush( stdout );

		NEW_PVM = TRUE;
	}

	pvm_setopt( PvmAutoErr, se );

	/* Get My TID */

	MYTID = pvm_mytid();

	if ( MYTID < 0 )
	{
		fprintf( stderr, "\n\nError connecting to pvmd: tid = %x\n\n",
			MYTID );
		
		exit( -1 );
	}

	else
		printf( "xpvm connected as tid %x.\n", MYTID );

	/* Set Desired Signals After pvm_mytid() - match new console */

	signal( SIGTERM, SIG_IGN );
	signal( SIGALRM, SIG_IGN );

	signal( SIGINT, (vfp) quit_proc );
	signal( SIGQUIT, (vfp) quit_proc );

	/* Set Up Host Add Notify */

	pvm_notify( PvmHostAdd, 99, -1, (int *) NULL );

	/* Set PVM Options */

	pvm_setopt( PvmTraceTid, MYTID );
	pvm_setopt( PvmTraceCode, 666 );

	pvm_setopt( PvmOutputTid, MYTID );
	pvm_setopt( PvmOutputCode, 667 );

	trace_mask_init();

	pvm_settmask( PvmTaskChild, TRACE_MASK );

	/* Set Up XPVM Group, Start Group Server (if necessary) */

	/* if ( Qflag ) */
		/* pvm_setopt( PvmCacheGS, 1 ); */

	inum = pvm_joingroup( "xpvm" );

	if ( inum != 0 )
	{
		printf( "\nWarning: XPVM Group Already Present, inum=%d\n\n",
			inum );
	}

	GROUPS_ALIVE = TRUE;

	read_hostfile();

	return( TCL_OK );
}

trace_mask_init()
{
	int j;

	/* Default Trace Mask */

	TEV_INIT_MASK( TRACE_MASK );

	/* Group Masks */

	for ( j=0 ; j < GROUP_MASK_MAX ; j++ )
		TEV_INIT_MASK( GROUP_MASKS[j] );

	/* Do All Events Case Hard-Wired */

	for ( j=TEV_FIRST ; j <= TEV_MAX ; j += 2 )
		TEV_SET_MASK( GROUP_MASKS[ GROUP_MASK_ALL_EVENTS ], j );

	TEV_UNSET_MASK( GROUP_MASKS[ GROUP_MASK_ALL_EVENTS ], TEV_NEWTASK );
	TEV_UNSET_MASK( GROUP_MASKS[ GROUP_MASK_ALL_EVENTS ], TEV_ENDTASK );
	TEV_UNSET_MASK( GROUP_MASKS[ GROUP_MASK_ALL_EVENTS ], TEV_SPNTASK );
}

window_init()
{
	static char fname[1024];

	FILE *fptest;

	/* Create Interface - TCL Script */

	fptest = fopen( "xpvm.tcl", "r" );

	if ( fptest != NULL )
	{
		strcpy( fname, "xpvm.tcl" );

		fclose( fptest );
	}

	else
		sprintf( fname, "%s/xpvm.tcl", XPVM_DIR );

	if ( Tcl_EvalFile( interp, fname ) == TCL_ERROR )
		return( TCL_ERROR );

	/* Check Hosts */

	initialize_hosts();

	/* Identify Any Existing Tasks */

	initialize_tasks();

	/* Initialize Trace Controls Setting */

	set_trace_controls();

	/* Set Up Event Polling Proc */

	Tk_DoWhenIdle( (Tk_IdleProc *) recv_event_proc, (ClientData) NULL );

	return( TCL_OK );
}

