
/* $Id: xpvm.h,v 2.310 1994/07/18 18:18:30 kohl Exp $ */

/* System Header Files */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>
#include <malloc.h>
#include <signal.h>
#include <pwd.h>
#include <math.h>

#define PI (M_PI)

/* Header Files */

#include <tk.h>

#include "const.h"

#include <pvm3.h>

#include <pvmtev.h>

#include "trace.h"

typedef void (*vfp)();
typedef int (*ifp)();

/* Program Data Structures */

struct msg_struct
{
	int src_tid;
	int dst_tid;
	int msgtag;
	int nbytes;
	struct timeval sendtime;
	struct timeval recvtime;
	/* MSG */ struct msg_struct *next;
};

typedef struct msg_struct *MSG;

struct task_struct
{
	char *name;
	int tid;
	int ptid;
	int pvmd_tid;
	int flags;
	int status;
	/* TASK */ struct task_struct *next;
};

typedef struct task_struct *TASK;

struct host_struct
{
	char *name;
	char *alias;
	char *refname;
	char *arch;
	int pvmd_tid;
	int in_pvm;
	int index;
	int speed;
	struct timeval delta;
	/* HOST */ struct host_struct *next;
};

typedef struct host_struct *HOST;

struct arch_struct
{
	char *code;
	char *name;
	int height;
	int width;
	/* ARCH */ struct arch_struct *next;
};

typedef struct arch_struct *ARCH;

struct trace_mask_index_struct
{
	char *name;
	int index;
	/* TRACE_MASK_INDEX */ struct trace_mask_index_struct *next;
};

typedef struct trace_mask_index_struct *TRACE_MASK_INDEX;

/* Routines */

MSG		create_msg();

TASK	create_task(), get_task_tid();

HOST	create_host(), get_host_index();

ARCH	create_arch(), get_arch_code();

TRACE_MASK_INDEX	create_trace_mask_index();

double	distance();
double	angle_of();

char	*assemble_menu_list();
char	*get_arch_icon_str();
char	*pvm_event_string();
char	*host_alias_str();
char	*center_str();
char	*check_arch();
char	*host_name();
char	*trunc_str();
char	*copy_str();
char	*date_str();
char	*pad_num();

FILE	*fopen();

/* Externals */

extern	char	*pvm_errlist[];

extern	int		pvm_nerr;

extern	int 	errno;

/* Global Variables */

#ifndef DO_GLOBALS

#include "tevfmt.h"

extern	struct tevfmt	tev_formats[TEV_MAX + 1];

extern	char	*GROUP_MASK_STRS[GROUP_MASK_MAX];

extern	char	*SIGNAL_STRS[NUM_SIGNALS];

extern	char	*HOST_CMD_STRS[MAX_HID];

extern	struct timeval	CURRENT_TIME;
extern	struct timeval	BASE_TIME;
extern	struct timeval	TRACE_TIME;

extern	int		PROCESS_COUNTER_OVERWRITE;
extern	int		PROCESS_COUNTER_PLAYBACK;

extern	int		RECV_TIMER_OVERWRITE;
extern	int		RECV_TIMER_PLAYBACK;

extern	int		PROCESS_COUNTER_INTERVAL;
extern	int		RECV_TIMER_INTERVAL;

extern	int		TRACE_OVERWRITE_FLAG;
extern	int		TRACE_FILE_STATUS;
extern	int		TRACE_GROUP_TASKS;
extern	int		TRACE_DO_SPAWN;
extern	int		TRACE_PENDING;
extern	int		TRACE_ACTIVE;
extern	int		TRACE_STATUS;

extern	Pvmtmask	TRACE_MASK;

extern	Pvmtmask	GROUP_MASKS[GROUP_MASK_MAX];

extern	int		TEV_TRACE[TRACE_MAX];

extern	int		TEV_TYPE[TRACE_MAX];

extern	char	TRACE_FILE[1024];

extern	FILE	*TRACE_OUT;
extern	FILE	*TRACE_IN;

extern	char	OUTPUT_FILE[1024];

extern	FILE	*OUTPUT_FP;

extern	int		GROUPS_ALIVE;

extern	Tcl_Interp	*interp;

extern	Tk_Window	Top;

extern	Display		*Disp;

extern	MSG		MSG_LIST;

extern	TASK	TASK_LIST;

extern	HOST	HOST_LIST;

extern	ARCH	ARCH_LIST;

extern	TRACE_MASK_INDEX	TM_INDEX_LIST;
extern	TRACE_MASK_INDEX	TM_INDEX_LAST;

extern	char	*HOST_NAME;

extern	char	*HOST_ALIAS;

extern	char	*HOME_DIR;

extern	char	*XPVM_DIR;

extern	char	*HOSTFILE;

extern	char	*screen_name;
extern	char	*base_name;

extern	int		NMSGS;

extern	int		NTASKS;

extern	int 	NHOSTS;

extern	int 	NARCHS;

extern	char	**Argv;
extern	int 	Argc;

extern	int		NEW_PVM;

extern	int 	MYTID;

extern	int 	DEPTH;

extern	int 	speed;

extern	int 	Qflag;
extern	int		Nflag;
extern	int		Xflag;
extern	int 	eflag;
extern	int 	tflag;
extern	int 	vflag;

#else

#include "tevfmt.c"

char	*GROUP_MASK_STRS[GROUP_MASK_MAX] =
{
	"ALL_EVENTS",
	"ALL_HOST_EVENTS",
	"ALL_GROUP_EVENTS",
	"ALL_TASK_EVENTS",
	"ALL_COMM_EVENTS",
	"ALL_COLL_EVENTS",
	"ALL_BUF_EVENTS",
	"ALL_PACK_EVENTS",
	"ALL_UNPACK_EVENTS",
	"ALL_INFO_EVENTS",
	"ALL_MISC_EVENTS"
};

char	*SIGNAL_STRS[NUM_SIGNALS] =
{
	"SIGHUP",
	"SIGINT",
	"SIGQUIT",
	"SIGILL",
	"SIGTRAP",
	"SIGABRT",
	"SIGEMT",
	"SIGFPE",
	"SIGKILL",
	"SIGBUS",
	"SIGSEGV",
	"SIGSYS",
	"SIGPIPE",
	"SIGALRM",
	"SIGTERM",
	"SIGURG",
	"SIGSTOP",
	"SIGTSTP",
	"SIGCONT",
	"SIGCHLD",
	"SIGTTIN",
	"SIGTTOU",
	"SIGIO",
	"SIGXCPU",
	"SIGXFSZ",
	"SIGVTALRM",
	"SIGPROF",
	"SIGWINCH",
	"SIGLOST",
	"SIGUSR1",
	"SIGUSR2"
};

char	*HOST_CMD_STRS[MAX_HID] =
{
	"Add All Hosts"
};

struct timeval	CURRENT_TIME;
struct timeval	BASE_TIME;
struct timeval	TRACE_TIME;

int		PROCESS_COUNTER_OVERWRITE;
int		PROCESS_COUNTER_PLAYBACK;

int		RECV_TIMER_OVERWRITE;
int		RECV_TIMER_PLAYBACK;

int		PROCESS_COUNTER_INTERVAL;
int		RECV_TIMER_INTERVAL;

int		TRACE_OVERWRITE_FLAG;
int		TRACE_FILE_STATUS;
int		TRACE_GROUP_TASKS;
int		TRACE_DO_SPAWN;
int		TRACE_PENDING;
int		TRACE_ACTIVE;
int		TRACE_STATUS;

Pvmtmask	TRACE_MASK;

Pvmtmask	GROUP_MASKS[GROUP_MASK_MAX];

int		TEV_TRACE[TRACE_MAX];

int		TEV_TYPE[TRACE_MAX];

char	TRACE_FILE[1024];

FILE	*TRACE_OUT;
FILE	*TRACE_IN;

char	OUTPUT_FILE[1024];

FILE	*OUTPUT_FP;

int		GROUPS_ALIVE;

Tcl_Interp	*interp;

Tk_Window	Top;

Display		*Disp;

MSG		MSG_LIST;

TASK	TASK_LIST;

HOST	HOST_LIST;

ARCH	ARCH_LIST;

TRACE_MASK_INDEX	TM_INDEX_LIST;
TRACE_MASK_INDEX	TM_INDEX_LAST;

char	*HOST_NAME;

char	*HOST_ALIAS;

char	*HOME_DIR;

char	*XPVM_DIR;

char	*HOSTFILE;

char	*screen_name = (char *) NULL;
char	*base_name = (char *) NULL;

int		NMSGS;

int		NTASKS;

int 	NHOSTS;

int 	NARCHS;

char	**Argv;
int 	Argc;

int		NEW_PVM;

int 	MYTID;

int 	DEPTH;

int 	speed;

int 	Qflag;
int		Nflag;
int		Xflag;
int 	eflag;
int 	tflag;
int 	vflag;

#endif

